# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that gets album information from the title
of a song via musicbrainz.

@author: Lateef Alabi-Oki
@organization: The striim Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class AlbumInfo(object):
	"""
	This class gets album information from the title of a song via
	musicbrainz.
	"""

	def __init__(self, manager, player):
		"""
		Initialize object.

		@param self: Reference to the AlbumInfo instance.
		@type self: An AlbumInfo object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__init_attributes(manager, player)
		self.__precompile_methods()
		self.__set_properties()
		self.__signal_id_1 = manager.connect("destroy", self.__destroy_cb)
		self.__signal_id_2 = player.connect("stream-update", self.__stream_update_cb)

	def __init_attributes(self, manager, player):
		"""
		Initialize data attributes.

		@param self: Reference to the AlbumInfo instance.
		@type self: An AlbumInfo object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__player = player
		self.__manager = manager
		from musicbrainz2.webservice import Query
		self.__query = Query()
		self.__signal_id_1 = self.__signal_id_2 = None
		return

	def __set_properties(self):
		"""
		Set properties.

		@param self: Reference to the AlbumInfo instance.
		@type self: An AlbumInfo object.
		"""
		if self.__player.title: self.__process_title(self.__player.title)
		return

	def __process_title(self, title):
		"""
		Process the title of a song.

		@param self: Reference to the AlbumInfo instance.
		@type self: An AlbumInfo object.

		@param title: The title of a song.
		@type title: A String object.
		"""
		try:
			from Exceptions import AlbumNameError
			artist, title = self.__get_info(title.strip())
			album = self.__get_album(artist, title)
			self.__player.emit("album-name-found", (artist, title, album))
		except AlbumNameError:
			self.__player.emit("no-album-name-found", (artist, title))
		except ValueError:
			self.__player.emit("no-album-name-found", ())
		return

	def __precompile_methods(self):
		"""
		Optimize methods via psyco.

		@param self: Reference to the AlbumInfo instance.
		@type self: An AlbumInfo object.
		"""
		try:
			from psyco import bind
			bind(self.__process_title)
			bind(self.__get_album)
			bind(self.__get_info)
			bind(self.__get_album_from)
		except ImportError:
			pass
		return

	def __get_info(self, title):
		"""
		Get artist and title information from title of a song.

		@param self: Reference to the AlbumInfo instance.
		@type self: An AlbumInfo object.

		@param title: The title of a song.
		@type title: A String object.

		@return: The artist and title of a song.
		@rtype: A String object.
		"""
		artist, title = title.split(" - ")
		artist = self.__filter_feat(artist)
		return artist, title

	def __filter_feat(self, artist):
		"""
		Remove name of artists that feature in a song to make album
		information retrieval easier.

		@param self: Reference to the AlbumInfo instance.
		@type self: An AlbumInfo object.

		@param artist: The artist of a song.
		@type artist: An String object.

		@param artist: The artist of a song.
		@type artist: A String object.
		"""
		words = artist.split()
		if "feat." in words:
			index = words.index("feat.")
			artist = " ".join(words[0:index])
		elif "ft." in words:
			index = words.index("ft.")
			artist = " ".join(words[0:index])
		elif "f." in words:
			index = words.index("f.")
			artist = " ".join(words[0:index])
		else:
			return artist.strip()
		return artist.strip()

	def __get_album(self, artist, title):
		"""
		Get album information from artist and title of a song.

		@param self: Reference to the AlbumInfo instance.
		@type self: An AlbumInfo object.

		@param artist: The artist of a song.
		@type artist: A String object.

		@param title: The title of a song.
		@type title: A String object.

		@return: The name of the album of a song.
		@rtype: A String object.
		"""
		from musicbrainz2.webservice import TrackFilter, WebServiceError
		try:
			f = TrackFilter(title=title, artistName=artist)
			results = self.__query.getTracks(f)
			album = self.__get_album_from(results)
		except WebServiceError, e:
			from Exceptions import AlbumNameError
			raise AlbumNameError
		return album

	def __get_album_from(self, results):
		"""
		Get album from results.

		@param self: Reference to the AlbumInfo instance.
		@type self: An AlbumInfo object.

		@param results: A musicbrainz data structure containing information of albums.
		@type results: A List object.

		@return: The album of a song.
		@rtype: A String object.
		"""
		from Exceptions import AlbumNameError
		if not results: raise AlbumNameError
		album = results[0].track.releases[0].title
		return album

	def __stream_update_cb(self, player, dictionary):
		"""
		Handles callback when the "stream-update" signal is emitted.

		@param self: Reference to the AlbumInfo instance.
		@type self: An AlbumInfo object.

		@param player: Reference to the Player instance.
		@type player: A Player object.

		@param dictionary: Metadata information about a song.
		@type dictionary: A Dictionary object.
		"""
		if dictionary.has_key("title"):
			self.__process_title(dictionary["title"])
		else:
			player.emit("no-album-name-found", ())
		return

	def __destroy_cb(self, *args):
		"""
		Handles callback when the "destroy" signal is emitted.

		@param self: Reference to the AlbumInfo instance.
		@type self: An AlbumInfo object.
		"""
		from Striim.utils import delete_attributes, disconnect_signal
		disconnect_signal(self.__signal_id_1, self.__manager)
		disconnect_signal(self.__signal_id_2, self.__player)
		delete_attributes(self)
		del self
		self = None
		return
