# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that defines the behavior of the combobox
in the station manager window.

@author: Lateef Alabi-Oki
@organization: The striim Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class ComboBox(object):
	"""
	This class defines the behavior of the combobox in the station
	manager.
	"""

	def __init__(self, manager, player):
		"""
		Initialize object.

		@param self: Reference to the ComboBox instance.
		@type self: A ComboBox object.

		@param manager: Reference to the manager.
		@type manager: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__init_attributes(manager, player)
		self.__set_properties()
		self.__update_genre_combobox()
		self.__set_genre_from_database()
		self.__signal_id_1 = manager.connect("destroy", self.__destroy_cb)
		self.__signal_id_2 = self.__combobox.connect("changed", self.__changed_cb)
		# Monitor database for changes.
		from gnomevfs import monitor_add, MONITOR_FILE
		self.__monitor_id = monitor_add(self.__database_uri, MONITOR_FILE, self.__database_changed_cb)


	def __init_attributes(self, manager, player):
		"""
		Initialize data attributes.

		@param self: Reference to the ComboBox instance.
		@type self: A ComboBox object.

		@param manager: Reference to the manager.
		@type manager: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__manager = manager
		self.__player = player
		self.__combobox = manager.glade.get_widget("GenreComboBox")
		self.__signal_id_1 = None
		# Path to the stations database.
		from Striim.Global import metadata_folder
		self.__database_path = metadata_folder + "/stations.gdb"
		from gnomevfs import get_uri_from_local_path
		self.__database_uri = get_uri_from_local_path(self.__database_path)
		self.__monitor_id = None
		return

	def __set_properties(self):
		"""
		Set default properties.

		@param self: Reference to the AddDialog instance.
		@type self: An AddDialog object.
		"""
		from gtk import ListStore, CellRendererText
		self.__combobox.set_property("model", ListStore(str))
		cell = CellRendererText()
		self.__combobox.pack_start(cell, True)
		self.__combobox.add_attribute(cell, 'text', 0)
		self.__combobox.get_model().append(["All"])
		self.__combobox.set_property("active", 0)
		self.__combobox.set_property("sensitive", False)
		return

	def __update_genre_combobox(self):
		"""
		Update the Genre combobox.

		@param self: Reference to the EditDialog instance.
		@type self: An EditDialog object.
		"""
		self.__combobox.set_property("sensitive", False)
		text = self.__get_active_text()
		from Metadata import get_stations_from_database
		stations = get_stations_from_database()
		model = self.__combobox.get_model()
		model.clear()
		model.append(["All"])
		genres = set([])
		from operator import not_
		if not_(stations): return self.__combobox.set_property("active", 0)
		for name, genre, uris in stations.values():
			if genre: genres.add(genre)
		genres = list(genres)
		genres.sort()
		boolean = False
		for genre in genres:
			boolean = True
			model.append([genre])
		self.__set_active_text(text)
		self.__combobox.set_property("sensitive", boolean)
		return

	def __get_active_text(self):
		"""
		Get current active text.

		@param self: Reference to the ComboBox instance.
		@type self: A ComboBox object.
		"""
		iterator = self.__combobox.get_active_iter()
		model = self.__combobox.get_model()
		text = model.get_value(iterator, 0)
		return text

	def __set_active_text(self, text):
		"""
		Determine active text and set it.

		@param self: Reference to the ComboBox instance.
		@type self: A ComboBox object.

		@param text: String representing a genre.
		@type text: A String object.
		"""
		model = self.__combobox.get_model()
		iterator = model.get_iter_first()
		from operator import not_, eq
		if not_(iterator): return
		while True:
			value = model.get_value(iterator, 0)
			if eq(value, text):
				self.__combobox.set_active_iter(iterator)
				return
			iterator = model.iter_next(iterator)
			if not_(iterator): break
		self.__combobox.set_property("active", 0)
		return

	def __set_genre_from_database(self):
		"""
		Determine active genre based on information in database.

		@param self: Reference to the ComboBox instance.
		@type self: A ComboBox object.
		"""
		from Metadata import get_genre_from_database
		text = get_genre_from_database()
		self.__set_active_text(text)
		self.__manager.emit("genre-changed", text)
		return

	def __destroy_cb(self, *args):
		"""
		Handles callback when the "destroy" signal is emitted.

		@param self: Reference to the ComboBox instance.
		@type self: A ComboBox object.
		"""
		from Striim.utils import delete_attributes, disconnect_signal
		disconnect_signal(self.__signal_id_1, self.__manager)
		disconnect_signal(self.__signal_id_2, self.__manager)
		from gnomevfs import monitor_cancel
		if self.__monitor_id: monitor_cancel(self.__monitor_id)
		self.__combobox.destroy()
		delete_attributes(self)
		del self
		self = None
		return

	def __changed_cb(self, *args):
		"""
		Handles callback when the "changed" signal is emitted.

		@param self: Reference to the ComboBox instance.
		@type self: A ComboBox object.
		"""
		text = self.__get_active_text()
		self.__manager.emit("genre-changed", text)
		from Metadata import update_genre_database
		update_genre_database(text)
		return

	def __database_changed_cb(self, monitor_uri, info_uri, event_type):
		"""
		Handles callback when the stations database is modified.

		@param self: Reference to the TemplateLoader instance.
		@type self: An TemplateLoader object.

		@param monitor_uri: The uri that is monitored.
		@type monitor_uri: A String object.

		@param info_uri: The uri that is monitored.
		@type info_uri: A String object.

		@param event_type: The type of modification that occured.
		@type event_type: A gnomevfs.MONITOR_EVENT* object.
		"""
		from gnomevfs import MONITOR_EVENT_DELETED
		from gnomevfs import MONITOR_EVENT_CREATED
		from gnomevfs import MONITOR_EVENT_CHANGED
		from operator import contains
		events = [MONITOR_EVENT_CHANGED, MONITOR_EVENT_DELETED, MONITOR_EVENT_CREATED]
		if contains(events, event_type):
			self.__update_genre_combobox()
		return
