# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that defines the behavior of the Station's
window.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class Window(object):
	"""
	This class defines the behavior of the Station's window.
	"""

	def __init__(self, manager, player):
		"""
		Initialize object.

		@param self: Reference to the Window instance.
		@type self: A Window object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__init_attributes(manager, player)
		self.__signal_id_1 = manager.connect("destroy", self.__destroy_cb)
		self.__signal_id_2 = manager.connect("show-window", self.__show_window_cb)
		self.__signal_id_3 = self.__window.connect("delete-event", self.__delete_event_cb)
		self.__signal_id_4 = self.__window.connect("key-press-event", self.__key_press_event_cb)
		self.__signal_id_5 = self.__window.connect("window-state-event", self.__window_state_event_cb)

	def __init_attributes(self, manager, player):
		"""
		Initialize data attributes.

		@param self: Reference to the Window instance.
		@type self: A Window object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__manager = manager
		self.__player = player
		self.__is_visible = False
		self.__is_maximized = False
		self.__window = manager.glade.get_widget("StationWindow")
		self.__signal_id_1 = self.__signal_id_2 = None
		self.__signal_id_3 = self.__signal_id_4 = None
		self.__signal_id_5 = None
		return

	def __show_window(self):
		"""
		Show window.

		@param self: Reference to the Window instance.
		@type self: A Window object.
		"""
		self.__player.response()
		from Metadata import get_position_from_database
		try:
			maximized, x, y, width, height = get_position_from_database()
			if maximized:
				self.__window.maximize()
			else:
				self.__window.move(x, y)
				self.__window.resize(width, height)
		except:
			pass
		self.__window.show_all()
		self.__window.present()
		self.__player.response()
		self.__manager.emit("showed-window")
		return

	def __hide_window(self):
		"""
		Hide window.

		@param self: Reference to the Window instance.
		@type self: A Window object.
		"""
		self.__player.response()
		from Metadata import update_position_in_database
		if self.__is_maximized:
			update_position_in_database((True, None, None, None, None))
		else:
			x, y = self.__window.get_position()
			width, height = self.__window.get_size()
			update_position_in_database((False, x, y, width, height))
		self.__window.hide()
		self.__player.response()
		return

	def __destroy_cb(self, *args):
		"""
		Handles callback when the "destroy" signal is emitted.

		@param self: Reference to the Window instance.
		@type self: A Window object.
		"""
		from Striim.utils import delete_attributes, disconnect_signal
		disconnect_signal(self.__signal_id_1, self.__manager)
		disconnect_signal(self.__signal_id_2, self.__manager)
		disconnect_signal(self.__signal_id_3, self.__window)
		disconnect_signal(self.__signal_id_4, self.__window)
		disconnect_signal(self.__signal_id_5, self.__window)
		self.__window.destroy()
		delete_attributes(self)
		del self
		self = None
		return

	def __show_window_cb(self, *args):
		"""
		Handles callback when the "show-window" signal is emitted.

		@param self: Reference to the Window instance.
		@type self: A Window object.
		"""
		self.__show_window()
		return

	def __delete_event_cb(self, *args):
		"""
		Handles callback when the "delete-event" signal is emitted.

		@param self: Reference to the Window instance.
		@type self: A Window object.
		"""
		self.__hide_window()
		return True

	def __key_press_event_cb(self, window, event):
		"""
		Handles callback when the "key-press-event" signal is emitted.

		@param self: Reference to the Window instance.
		@type self: A Window object.

		@param window: Reference to the Window instance.
		@type window: A Window object.

		@param event: An event.
		@type event: A gtk.Event object.
		"""
		from gtk import keysyms
		from operator import eq
		# Disable tabbing.
		if eq(event.keyval, keysyms.Escape): self.__hide_window()
		return False

	def __window_state_event_cb(self, window, event):
		"""
		Handles callback when the "window-state-event" signal is emitted.
		
		@param self: Reference to the Window instance.
		@type self: A Window object.
		
		@param window: Reference to the Window instance.
		@type window: A Window object.
		
		@param event: An event.
		@type event: A gtk.Event object.
		"""
		from operator import eq
		from gtk.gdk import WINDOW_STATE_MAXIMIZED, WINDOW_STATE_ICONIFIED
		if eq(event.new_window_state, WINDOW_STATE_ICONIFIED): return False
		self.__is_maximized = False
		if eq(event.new_window_state, WINDOW_STATE_MAXIMIZED):
			self.__is_maximized = True
		return False
