# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents the class that defines the volume range behavior.

@author: Lateef Alabi-Oki
@organization: The striim Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class VolumeRange(object):
	"""
	This class defines the behavior of the volume range.
	"""

	def __init__(self, manager, glade, feedback):
		"""
		Initialize object.

		@param self: Reference to the VolumeRange instance.
		@type self: A VolumeRange object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__init_attributes(manager, glade, feedback)
		self.__sig_id_1 = manager.connect("destroy", self.__destroy_cb)
		self.__sig_id_2 = manager.connect("increase-volume", self.__increase_volume_cb)
		self.__sig_id_3 = manager.connect("decrease-volume", self.__decrease_volume_cb)
		self.__sig_id_4 = manager.connect("volume", self.__volume_cb)
		self.__sig_id_5 = self.__range.connect("value-changed", self.__value_changed_cb)

	def __init_attributes(self, manager, glade, feedback):
		"""
		Initialize data attributes.

		@param self: Reference to the VolumeRange instance.
		@type self: A VolumeRange object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__manager = manager
		self.__feedback = feedback
		self.__range = glade.get_widget("VolumeRange")
		return

	def __set_value(self, data):
		"""
		Set new value of range.

		@param self: Reference to the VolumeRange instance.
		@type self: A VolumeRange object.

		@param data: New volume level.
		@type data: A Float object.
		"""
		value = self.__range.get_value()
		from operator import eq
		if eq(value, data): return
		self.__range.set_value(data)
		return

	def __update_volume_database(self):
		"""
		Update the volume database.

		@param self: Reference to the VolumeRange instance.
		@type self: A VolumeRange object.
		"""
		from Metadata import update_volume_database
		update_volume_database(round(self.__range.get_value(), 1))
		return False

	def __value_changed_cb(self, *args):
		"""
		Handles callback when the "value-changed" signal is emitted.

		@param self: Reference to the VolumeRange instance.
		@type self: A VolumeRange object.
		"""
		from gobject import idle_add, source_remove
		try:
			source_remove(self.__update_volume_database_id)
		except:
			pass
		self.__update_volume_database_id = idle_add(self.__update_volume_database)
		return

	def __increase_volume_cb(self, *args):
		"""
		Handles callback when the "increase-volume" signal is emitted.

		@param self: Reference to the VolumeRange instance.
		@type self: A VolumeRange object.
		"""
		value = self.__range.get_value()
		data = round(value + 0.1, 1)
		feedback = self.__feedback
		from operator import gt
		from i18n import msg0001, msg0002
		if gt(data, 1):
			feedback.update_status_message(msg0001, "warning")
		else:
			self.__set_value(data)
			feedback.update_status_message(msg0002, "yes")
		return

	def __decrease_volume_cb(self, *args):
		"""
		Handles callback when the "decrease-volume" signal is emitted.

		@param self: Reference to the VolumeRange instance.
		@type self: A VolumeRange object.
		"""
		value = self.__range.get_value()
		data = round(value - 0.1, 1)
		feedback = self.__feedback
		from operator import lt
		from i18n import msg0004, msg0003
		if lt(data, 0):
			feedback.update_status_message(msg0003, "warning")
		else:
			self.__set_value(data)
			feedback.update_status_message(msg0004, "yes")
		return

	def __volume_cb(self, manager, volume, data):
		"""
		Handles callback when the volume level changes.

		@param self: Reference to the VolumeRange instance.
		@type self: A VolumeRange object.

		@param volume: New volume level.
		@type volume: A Float object.

		@param initial_value: If volume is got at initialization.
		@type initial_value: A Boolean object.
		"""
		from operator import not_
		if not_(data): return
		self.__range.set_property("sensitive", True)
		self.__set_value(volume)
		return

	def __destroy_cb(self, *args):
		"""
		Handles callback when the "destroy" signal is emitted.

		@param self: Reference to the VolumeRange instance.
		@type self: A VolumeRange object.
		"""
		from Striim.utils import delete_attributes, disconnect_signal
		disconnect_signal(self.__sig_id_1, self.__manager)
		disconnect_signal(self.__sig_id_2, self.__manager)
		disconnect_signal(self.__sig_id_3, self.__manager)
		disconnect_signal(self.__sig_id_4, self.__manager)
		disconnect_signal(self.__sig_id_5, self.__range)
		self.__range.destroy()
		delete_attributes(self)
		del self
		self = None
		return
